@@ -17,7 +17,7 @@
 //!   implementations).
 //!
 //! ## Example
-//! ```
+//! ```ignore
 //! use fixnum::{FixedPoint, typenum::U9, ops::{CheckedAdd, RoundingMul, RoundMode::*, Zero}};
 //!
 //! /// Signed fixed point amount over 64 bits, 9 decimal places.
@@ -63,7 +63,7 @@
 //! ## Implementing wrapper types.
 //! It's possible to restrict the domain in order to reduce chance of mistakes.
 //! Note that convenient [`fixnum!` macro][fixnum] works with wrapper types too.
-//! ```
+//! ```ignore
 //! use derive_more::From;
 //! use fixnum::{impl_op, typenum::U9, FixedPoint, fixnum};
 //!
@@ -128,8 +128,6 @@ use core::convert::{TryFrom, TryInto};
 use core::str::FromStr;
 use core::{fmt, i64, marker::PhantomData};
 
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
 use typenum::Unsigned;
 
 #[cfg(feature = "i128")]
@@ -142,15 +140,22 @@ mod errors;
 #[cfg(feature = "i128")]
 mod i256;
 mod macros;
-pub mod ops;
 #[cfg(feature = "parity")]
 mod parity;
 mod power_table;
 #[cfg(test)]
 mod tests;
 
+#[cfg(not(any(feature = "i16", feature = "i32", feature = "i64", feature = "i128")))]
+compile_error!("Some of the next features must be enabled: \"i128\", \"i64\", \"i32\", \"i16\"");
+
 pub use errors::*;
 
+pub mod ops;
+#[cfg(all(feature = "serde", feature = "std"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "std", feature = "serde"))))]
+pub mod serde;
+
 #[doc(hidden)]
 pub mod _priv {
     pub use crate::const_fn::*;
@@ -176,8 +181,11 @@ type Result<T, E = ArithmeticError> = core::result::Result<T, E>;
 /// MAX = (2 ^ (64 - 1) - 1) / 1e9 = 9223372036.854775807 ~ 9.2e9
 /// ERROR_MAX = 0.5 / 1e9 = 5e-10
 /// ```
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Clone, Copy, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(feature = "i128", feature = "i64", feature = "i32", feature = "i16")))
+)]
 pub struct FixedPoint<I, P> {
     inner: I,
     _marker: PhantomData<P>,
@@ -466,7 +474,7 @@ macro_rules! impl_fixed_point {
 
             /// Takes [rounded][RoundMode] integral part of the number.
             ///
-            /// ```
+            /// ```ignore
             /// use fixnum::{FixedPoint, typenum::U9, ops::RoundMode::*};
             ///
             /// type Amount = FixedPoint<i64, U9>;
@@ -532,17 +540,14 @@ macro_rules! impl_fixed_point {
 
             #[cfg(feature = "std")]
             #[cfg_attr(docsrs, doc(cfg(feature = "std")))]
+            #[deprecated(since = "0.6.0", note = "Use `TryFrom` instead")]
             pub fn rounding_from_f64(value: f64) -> Result<FixedPoint<$layout, P>> {
-                let x = (value * Self::COEF as f64).round();
-                if x >= ($layout::MIN as f64) && x <= ($layout::MAX as f64) {
-                    Ok(Self::from_bits(x as $layout))
-                } else {
-                    Err(ArithmeticError::Overflow)
-                }
+                value.try_into().map_err(|_| ArithmeticError::Overflow)
             }
 
+            #[deprecated(since = "0.6.0", note = "Use `From` instead")]
             pub fn to_f64(self) -> f64 {
-                (self.inner as f64) / (Self::COEF as f64)
+                self.into()
             }
 
             // TODO: make this operation checked
@@ -605,9 +610,9 @@ macro_rules! impl_fixed_point {
             pub fn from_decimal(
                 mantissa: $layout,
                 exponent: i32,
-            ) -> Result<FixedPoint<$layout, P>, FromDecimalError> {
+            ) -> Result<FixedPoint<$layout, P>, ConvertError> {
                 if exponent < -Self::PRECISION || exponent > 10 {
-                    return Err(FromDecimalError::UnsupportedExponent);
+                    return Err(ConvertError::new("unsupported exponent"));
                 }
 
                 let ten: $layout = 10;
@@ -616,7 +621,48 @@ macro_rules! impl_fixed_point {
                 mantissa
                     .checked_mul(multiplier)
                     .map(Self::from_bits)
-                    .map_or_else(|| Err(FromDecimalError::TooBigMantissa), Ok)
+                    .map_or_else(|| Err(ConvertError::new("too big mantissa")), Ok)
+            }
+        }
+
+        impl<P: Precision> From<FixedPoint<$layout, P>> for f64 {
+            fn from(value: FixedPoint<$layout, P>) -> Self {
+                let coef = FixedPoint::<$layout, P>::COEF;
+                let integral = (value.inner / coef) as f64;
+                let fractional = ((value.inner % coef) as f64) / (coef as f64);
+                integral + fractional
+            }
+        }
+
+        #[cfg(feature = "std")]
+        impl<P: Precision> TryFrom<f64> for FixedPoint<$layout, P> {
+            type Error = ConvertError;
+
+            // TODO: it's a baseline implementation. See #18.
+            fn try_from(value: f64) -> Result<Self, Self::Error> {
+                use std::io::Write;
+
+                if !value.is_finite() {
+                    return Err(ConvertError::new("not finite"));
+                }
+
+                let mut buffer = [0u8; 64];
+
+                let abs = value.abs();
+
+                // f64 seems to have at least 15 significant decimal digits.
+                let prec = 15usize.saturating_sub(abs.log10().ceil() as usize)
+                    .min(Self::PRECISION as usize)
+                    .max(0);
+
+                write!(&mut buffer[..], "{:.*}", prec, value)
+                    .map_err(|_| ConvertError::new("too big number"))?;
+
+                let s = std::str::from_utf8(&buffer)
+                    .map_err(|_| ConvertError::new("not finite"))?
+                    .trim_end_matches('\0');
+
+                s.parse().map_err(|_| ConvertError::new("not finite"))
             }
         }
 
@@ -627,10 +673,10 @@ macro_rules! impl_fixed_point {
 
                 fn try_from(value: $try_from) -> Result<Self, Self::Error> {
                     $layout::try_from(value)
-                        .map_err(|_| ConvertError::Overflow)?
+                        .map_err(|_| ConvertError::new("too big number"))?
                         .checked_mul(Self::COEF)
                         .map(Self::from_bits)
-                        .ok_or(ConvertError::Overflow)
+                        .ok_or(ConvertError::new("too big number"))
                 }
             }
         )*
@@ -647,69 +693,76 @@ macro_rules! impl_fixed_point {
                     Some(index) => index,
                     None => {
                         let integral: $layout = str.parse().map_err(|_| {
-                            ConvertError::Other("can't parse integral part of the str")
+                            ConvertError::new("can't parse integral part of the str")
                         })?;
                         return integral
                             .checked_mul(coef)
-                            .ok_or(ConvertError::Overflow)
+                            .ok_or(ConvertError::new("too big integral part"))
                             .map(Self::from_bits);
                     }
                 };
 
                 let integral: $layout = str[0..index]
                     .parse()
-                    .map_err(|_| ConvertError::Other("can't parse integral part"))?;
+                    .map_err(|_| ConvertError::new("can't parse integral part"))?;
                 let fractional_str = &str[index + 1..];
 
                 if !fractional_str.chars().all(|c| c.is_digit(10)) {
-                    return Err(ConvertError::Other("can't parse fractional part: must contain digits only"));
+                    return Err(ConvertError::new("can't parse fractional part: must contain digits only"));
                 }
 
                 if fractional_str.len() > Self::PRECISION.abs() as usize {
-                    return Err(ConvertError::Other("requested precision is too high"));
+                    return Err(ConvertError::new("requested precision is too high"));
                 }
 
                 let ten: $layout = 10;
                 let exp = ten.pow(fractional_str.len() as u32);
 
                 if exp > coef {
-                    return Err(ConvertError::Other("requested precision is too high"));
+                    return Err(ConvertError::new("requested precision is too high"));
                 }
 
                 let fractional: $layout = fractional_str.parse().map_err(|_| {
-                    ConvertError::Other("can't parse fractional part")
+                    ConvertError::new("can't parse fractional part")
                 })?;
 
-                let final_integral = integral.checked_mul(coef).ok_or(ConvertError::Overflow)?;
+                let final_integral = integral.checked_mul(coef).ok_or(ConvertError::new("too big integral"))?;
                 let signum = if str.as_bytes()[0] == b'-' { -1 } else { 1 };
                 let final_fractional = signum * coef / exp * fractional;
 
                 final_integral
                     .checked_add(final_fractional)
                     .map(Self::from_bits)
-                    .ok_or(ConvertError::Overflow)
+                    .ok_or(ConvertError::new("too big number"))
             }
         }
     };
 }
 
+#[cfg(any(feature = "i64", feature = "i32", feature = "i16"))]
 const fn identity<T>(x: T) -> T {
     x
 }
 
+#[cfg(feature = "i16")]
 impl_fixed_point!(
+    #[cfg_attr(docsrs, doc(cfg(feature = "i16")))]
     inner = i16;
     promoted_to = i32;
     convert = identity;
     try_from = [i8, u8, i16, u16, i32, u32, i64, u64, i128, u128, isize, usize];
 );
+#[cfg(feature = "i32")]
 impl_fixed_point!(
+    #[cfg_attr(docsrs, doc(cfg(feature = "i32")))]
     inner = i32;
     promoted_to = i64;
     convert = identity;
     try_from = [i8, u8, i16, u16, i32, u32, i64, u64, i128, u128, isize, usize];
 );
+#[cfg(feature = "i64")]
 impl_fixed_point!(
+    #[cfg_attr(docsrs, doc(cfg(feature = "i64")))]
     inner = i64;
     promoted_to = i128;
     convert = identity;
