@@ -32,6 +32,9 @@ fn display() -> Result<()> {
     test_fixed_point! {
         case (x | FixedPoint, expected | &str) => {
             assert_eq!(format!("{}", x), String::from(expected));
+
+            #[cfg(feature = "serde")]
+            assert_eq!(serde_json::to_string(&x).unwrap(), format!("\"{}\"", expected));
         },
         all {
             (fp!(0), "0.0");
@@ -39,34 +42,61 @@ fn display() -> Result<()> {
             (fp!(-10.042), "-10.042");
             (fp!(0.000000001), "0.000000001");
             (fp!(-0.000000001), "-0.000000001");
+            (fp!(9223372036.854775807), "9223372036.854775807");
+            (fp!(-9223372036.854775808), "-9223372036.854775808");
         },
         fp128 {
             (fp!(0.000000000000000001), "0.000000000000000001");
             (fp!(-0.000000000000000001), "-0.000000000000000001");
+            (fp!(170141183460469231731.687303715884105727), "170141183460469231731.687303715884105727");
+            (fp!(-170141183460469231731.687303715884105728), "-170141183460469231731.687303715884105728");
         },
     };
     Ok(())
 }
 
 #[test]
+#[allow(overflowing_literals)]
 fn from_good_str() -> Result<()> {
     test_fixed_point! {
-        case (input | &str, expected | &str) => {
+        case (input | &str, expected | Layout) => {
+            let expected = FixedPoint::from_bits(expected);
             let input: FixedPoint = input.parse()?;
-            let expected: FixedPoint = expected.parse()?;
             assert_eq!(input, expected);
+
+            #[cfg(all(feature = "std", feature = "serde"))]
+            assert_eq!(
+                serde_json::from_str::<FixedPoint>(&format!("\"{}\"", input)).unwrap(),
+                expected
+            );
         },
-        all {
-            ("1", "1.000000");
-            ("1", "1.000000000");
-            ("1.1", "1.100000000");
-            ("1.02", "1.020000000");
-            ("-1.02", "-1.020000000");
-            ("+1.02", "1.020000000");
-            ("0.1234", "0.123400000");
-            ("-0.1234", "-0.123400000");
-            ("123456789.123456789", "123456789.123456789");
-            ("9223372036.854775807", "9223372036.854775807");
+        fp64 {
+            ("1", 1000000000);
+            ("1.1", 1100000000);
+            ("1.02", 1020000000);
+            ("-1.02", -1020000000);
+            ("+1.02", 1020000000);
+            ("0.1234", 123400000);
+            ("-0.1234", -123400000);
+            ("123456789.123456789", 123456789123456789);
+            ("9223372036.854775807", 9223372036854775807);
+            ("-9223372036.854775808", -9223372036854775808);
+        },
+        fp128 {
+            ("1", 1000000000000000000);
+            ("1.1", 1100000000000000000);
+            ("1.02", 1020000000000000000);
+            ("-1.02", -1020000000000000000);
+            ("+1.02", 1020000000000000000);
+            ("0.1234", 123400000000000000);
+            ("-0.1234", -123400000000000000);
+            ("123456789.123456789", 123456789123456789000000000);
+            ("9223372036.854775807", 9223372036854775807000000000);
+            ("-9223372036.854775808", -9223372036854775808000000000);
+            ("170141183460469231731.687303715884105727",
+             170141183460469231731687303715884105727);
+            ("-170141183460469231731.687303715884105728",
+             -170141183460469231731687303715884105728);
         },
     };
     Ok(())
@@ -78,6 +108,9 @@ fn from_bad_str() -> Result<()> {
         case (bad_str | &str) => {
             let result: Result<FixedPoint, ConvertError> = bad_str.parse();
             assert!(result.is_err(), "must not parse '{}'", bad_str);
+
+            #[cfg(all(feature = "std", feature = "serde"))]
+            assert!(serde_json::from_str::<FixedPoint>(&format!("\"{}\"", bad_str)).is_err());
         },
         all {
             ("");
@@ -100,6 +133,32 @@ fn from_bad_str() -> Result<()> {
     Ok(())
 }
 
+#[test]
+#[cfg(all(feature = "std", feature = "serde"))]
+fn serde_with() -> Result<()> {
+    test_fixed_point! {
+        case (input | f64, expected | FixedPoint) => {
+            #[derive(::serde::Serialize, ::serde::Deserialize)]
+            struct Struct {
+                #[serde(with = "crate::serde::as_f64")]
+                number: FixedPoint,
+            }
+
+            let actual = serde_json::from_str::<Struct>(&format!(r#"{{"number":{}}}"#, input)).unwrap().number;
+            assert_eq!(expected, actual);
+        },
+        all {
+            (1., fp!(1.0));
+            (1.1, fp!(1.1));
+            (1.02, fp!(1.02));
+            (-1.02, fp!(-1.02));
+            (0.1234, fp!(0.1234));
+            (-0.1234, fp!(-0.1234));
+        },
+    };
+    Ok(())
+}
+
 #[test]
 #[allow(clippy::assertions_on_constants)]
 fn exp_and_coef_should_agree() -> Result<()> {
@@ -634,18 +693,83 @@ fn rounding_to_i64() -> Result<()> {
 
 #[test]
 #[cfg(feature = "std")]
+#[allow(clippy::float_cmp)]
 fn to_f64() -> Result<()> {
     test_fixed_point! {
         case (x | FixedPoint, expected | f64) => {
-            assert_eq!(x.to_f64().to_string(), expected.to_string());
+            assert_eq!(f64::from(x), expected);
         },
         all {
             (fp!(0), 0.0);
+            (fp!(0.1), 0.1);
             (fp!(1), 1.0);
             (fp!(1.5), 1.5);
-            (fp!(42.123456789), 42.123_456_789);
+            (fp!(-5), -5.);
             (fp!(-14.14), -14.14);
-            (fp!(8003332421.536753168), 8_003_332_421.536_754);
+            (fp!(42.123456789), 42.123456789);
+            (fp!(-42.123456789), -42.123456789);
+            (fp!(8003332421.536753168), 8003332421.536753);
+            (fp!(-8003332421.536753168), -8003332421.536753);
+            (fp!(9223372036.854775807), 9223372036.854776);
+            (fp!(-9223372036.854775807), -9223372036.854776);
+            (fp!(922337203.685477581), 922337203.6854776);
+            (fp!(-922337203.685477581), -922337203.6854776);
+            (fp!(92233720.36854775), 92233720.36854775);
+            (fp!(-92233720.36854775), -92233720.36854775);
+        },
+        fp128 {
+            (fp!(0.0000000000025), 25e-13);
+            (fp!(1000000.0000000000025), 1e6);
+            (fp!(-1000000.0000000000025), -1e6);
+            (fp!(0.000000000000000025), 25e-18);
+            (fp!(-0.000000000000000025), -25e-18);
+            (fp!(2.1234567890123457), 2.1234567890123457);
+            (fp!(-2.1234567890123457), -2.1234567890123457);
+        },
+    };
+    Ok(())
+}
+
+#[test]
+#[cfg(feature = "std")]
+#[allow(clippy::float_cmp)]
+fn from_f64() -> Result<()> {
+    test_fixed_point! {
+        case (x | FixedPoint, expected | f64) => {
+            assert_eq!(Ok(x), expected.try_into());
+            assert_eq!(Ok(x.cneg().unwrap()), (-expected).try_into());
+        },
+        all {
+            (fp!(0), 0.0);
+            (fp!(0.5), 0.5);
+            (fp!(1), 1.0);
+            (fp!(1.5), 1.5);
+            (fp!(42.123456789), 42.123456789);
+
+            (fp!(803332.421536753), 803332.421536753);
+            (fp!(8033324.21536753), 8033324.21536753);
+            (fp!(80333242.1536753), 80333242.1536753);
+            (fp!(803332421.536753), 803332421.536753);
+            (fp!(8033324215.36753), 8033324215.36753);
+
+            (fp!(9223372036.85477), 9223372036.85477);
+        },
+        fp128 {
+            (fp!(0.803332421536753), 0.803332421536753);
+            (fp!(8.03332421536753), 8.03332421536753);
+            (fp!(8.03332421536753), 8.03332421536753);
+            (fp!(80.3332421536753), 80.3332421536753);
+            (fp!(803.332421536753), 803.332421536753);
+            (fp!(8033.32421536753), 8033.32421536753);
+            (fp!(80333.2421536753), 80333.2421536753);
+            // <see part of cases in `all` sections>
+            (fp!(80333242153.6753), 80333242153.6753);
+            (fp!(803332421536.753), 803332421536.753);
+            (fp!(8033324215367.53), 8033324215367.53);
+            (fp!(80333242153675.3), 80333242153675.3);
+            (fp!(803332421536753), 803332421536753.);
+            (fp!(8033324215367530), 8033324215367530.);
+            (fp!(8033324215367533), 8033324215367533.);
         },
     };
     Ok(())
